// Database Schema and Connection for SupplyChain Pulse Subscription System
const { MongoClient, ObjectId } = require('mongodb');
const logger = require('../utils/logger');

// MongoDB connection string
const uri = process.env.MONGODB_URI || 'mongodb://localhost:27017/supplychainpulse';

// Database and collections
let db;
let collections = {};

// Initialize database connection
async function connect() {
  try {
    const client = new MongoClient(uri, {
      useNewUrlParser: true,
      useUnifiedTopology: true
    });
    
    await client.connect();
    logger.info('Connected to MongoDB');
    
    db = client.db();
    
    // Initialize collections
    collections.users = db.collection('users');
    collections.plans = db.collection('plans');
    collections.subscriptions = db.collection('subscriptions');
    collections.payments = db.collection('payments');
    
    // Create indexes
    await collections.users.createIndex({ email: 1 }, { unique: true });
    await collections.subscriptions.createIndex({ userId: 1 });
    await collections.subscriptions.createIndex({ stripeSubscriptionId: 1 });
    await collections.subscriptions.createIndex({ paypalSubscriptionId: 1 });
    await collections.payments.createIndex({ userId: 1 });
    
    return db;
  } catch (error) {
    logger.error('MongoDB connection error:', error);
    process.exit(1);
  }
}

// Helper to convert string IDs to ObjectId
function toObjectId(id) {
  try {
    return new ObjectId(id);
  } catch (error) {
    return null;
  }
}

// Export database and collections
module.exports = {
  connect,
  toObjectId,
  get db() {
    return db;
  },
  get users() {
    return collections.users;
  },
  get plans() {
    return collections.plans;
  },
  get subscriptions() {
    return collections.subscriptions;
  },
  get payments() {
    return collections.payments;
  }
};
