# SupplyChain Pulse Subscription System Integration Guide

This document provides comprehensive instructions for integrating the subscription payment system with the SupplyChain Pulse website. The system supports both Stripe and PayPal payment processors, giving customers flexibility in how they pay for their subscriptions.

## Table of Contents

1. [System Overview](#system-overview)
2. [Prerequisites](#prerequisites)
3. [Installation](#installation)
4. [Configuration](#configuration)
5. [Integration with Main Website](#integration-with-main-website)
6. [Security Considerations](#security-considerations)
7. [Testing](#testing)
8. [Deployment](#deployment)
9. [Maintenance](#maintenance)

## System Overview

The subscription payment system consists of the following components:

- **Frontend Components**:
  - Subscription page with plan selection
  - Payment forms for both Stripe and PayPal
  - User account management interface
  - Subscription management interface

- **Backend Components**:
  - User management API
  - Stripe payment processing API
  - PayPal payment processing API
  - Database schema and connection
  - Authentication middleware
  - Logging utilities

- **Database Collections**:
  - Users
  - Plans
  - Subscriptions
  - Payments

## Prerequisites

Before installation, ensure you have the following:

- Node.js (v14 or higher)
- MongoDB (v4.4 or higher)
- Stripe account with API keys
- PayPal Business account with API keys
- SSL certificate for secure payment processing

## Installation

1. **Clone the repository**:
   ```
   git clone https://github.com/your-organization/supplychainpulse-payment-system.git
   cd supplychainpulse-payment-system
   ```

2. **Install dependencies**:
   ```
   npm install
   ```

3. **Create environment file**:
   Create a `.env` file in the root directory with the following variables:
   ```
   NODE_ENV=production
   PORT=3000
   MONGODB_URI=mongodb://localhost:27017/supplychainpulse
   JWT_SECRET=your_jwt_secret_key
   STRIPE_SECRET_KEY=your_stripe_secret_key
   STRIPE_PUBLISHABLE_KEY=your_stripe_publishable_key
   STRIPE_WEBHOOK_SECRET=your_stripe_webhook_secret
   PAYPAL_CLIENT_ID=your_paypal_client_id
   PAYPAL_CLIENT_SECRET=your_paypal_client_secret
   ```

4. **Initialize database**:
   ```
   node scripts/init-db.js
   ```

## Configuration

### Stripe Configuration

1. Log in to your Stripe Dashboard
2. Navigate to Developers > API keys
3. Copy your Publishable key and Secret key to the `.env` file
4. Set up webhook endpoint:
   - Go to Developers > Webhooks
   - Add endpoint: `https://your-domain.com/api/stripe/webhook`
   - Select events: `customer.subscription.created`, `customer.subscription.updated`, `customer.subscription.deleted`, `invoice.payment_succeeded`, `invoice.payment_failed`
   - Copy the Signing Secret to the `.env` file

### PayPal Configuration

1. Log in to your PayPal Developer Dashboard
2. Navigate to My Apps & Credentials
3. Create a new app or select an existing one
4. Copy the Client ID and Secret to the `.env` file
5. Set up IPN (Instant Payment Notification):
   - Go to Account Settings > Notifications
   - Update IPN settings with URL: `https://your-domain.com/api/paypal/ipn`

## Integration with Main Website

### Frontend Integration

1. **Add subscription link to main navigation**:
   ```html
   <a href="/subscription" class="nav-link">Pricing & Subscription</a>
   ```

2. **Include subscription CSS**:
   Add the following to your main website's head section:
   ```html
   <link rel="stylesheet" href="/css/subscription.css">
   ```

3. **Include subscription JavaScript**:
   Add the following before the closing body tag:
   ```html
   <script src="https://js.stripe.com/v3/"></script>
   <script src="https://www.paypal.com/sdk/js?client-id=YOUR_PAYPAL_CLIENT_ID&currency=USD"></script>
   <script src="/js/subscription.js"></script>
   ```

### Backend Integration

1. **Mount the subscription API routes**:
   In your main server.js file:
   ```javascript
   // Subscription system routes
   app.use('/api/users', require('./payment_system/user_management/user-api'));
   app.use('/api/stripe', require('./payment_system/stripe/stripe-api'));
   app.use('/api/paypal', require('./payment_system/paypal/paypal-api'));
   ```

2. **Add authentication middleware**:
   ```javascript
   const auth = require('./payment_system/middleware/auth');
   
   // Protected route example
   app.get('/dashboard', auth.requireAuth, (req, res) => {
     // Only authenticated users can access
     res.sendFile(path.join(__dirname, 'public', 'dashboard.html'));
   });
   ```

3. **Integrate user sessions**:
   Ensure your main website's authentication system works with the subscription system's JWT tokens.

## Security Considerations

1. **PCI Compliance**:
   - Never store credit card information
   - Use Stripe Elements to securely collect payment information
   - Ensure SSL is properly configured

2. **API Key Security**:
   - Keep all API keys in environment variables
   - Never expose secret keys in client-side code
   - Use different API keys for development and production

3. **Data Protection**:
   - Encrypt sensitive user data
   - Implement proper access controls
   - Regularly backup the database

4. **CSRF Protection**:
   - Implement CSRF tokens for all forms
   - Validate origin of requests

5. **Rate Limiting**:
   - Implement rate limiting for authentication endpoints
   - Protect against brute force attacks

## Testing

1. **Test Stripe Integration**:
   - Use Stripe test keys for development
   - Test card: 4242 4242 4242 4242 (any future expiry date, any CVC)
   - Test subscription creation, updates, and cancellation

2. **Test PayPal Integration**:
   - Use PayPal sandbox accounts
   - Test payment flow and IPN handling
   - Verify subscription management

3. **Test User Management**:
   - Registration and login
   - Password reset flow
   - Profile updates

4. **Test Subscription Management**:
   - Plan changes
   - Cancellations
   - Payment method updates

## Deployment

1. **Prepare for deployment**:
   ```
   npm run build
   ```

2. **Deploy to Namecheap Stellar Hosting**:
   - Upload all files to your hosting account
   - Set up MongoDB connection (use MongoDB Atlas for cloud hosting)
   - Configure environment variables in hosting control panel
   - Set up SSL certificate

3. **Verify deployment**:
   - Test all functionality in production environment
   - Verify webhook endpoints are accessible
   - Check database connections

## Maintenance

1. **Regular updates**:
   - Keep dependencies updated
   - Apply security patches promptly

2. **Monitoring**:
   - Set up logging and error tracking
   - Monitor subscription events and payment failures
   - Set up alerts for critical errors

3. **Backup strategy**:
   - Regular database backups
   - Secure storage of backup files

4. **Support procedures**:
   - Document common customer issues
   - Create support response templates
   - Set up customer support email

---

For any questions or issues, please contact the development team at support@supplychainpulse.com.
