// Authentication Middleware for SupplyChain Pulse Subscription System
const jwt = require('jsonwebtoken');
const db = require('../database/db');
const logger = require('../utils/logger');

// Middleware to require authentication
const requireAuth = async (req, res, next) => {
  try {
    // Get token from header
    const token = req.header('x-auth-token');
    
    // Check if no token
    if (!token) {
      return res.status(401).json({
        success: false,
        error: 'No authentication token, access denied'
      });
    }
    
    // Verify token
    const decoded = jwt.verify(token, process.env.JWT_SECRET);
    
    // Find user
    const user = await db.users.findOne({ _id: db.toObjectId(decoded.id) });
    if (!user) {
      return res.status(401).json({
        success: false,
        error: 'User not found'
      });
    }
    
    // Add user to request
    req.user = user;
    next();
  } catch (error) {
    logger.error('Authentication error:', error);
    res.status(401).json({
      success: false,
      error: 'Invalid authentication token'
    });
  }
};

// Middleware to require admin role
const requireAdmin = async (req, res, next) => {
  try {
    // First authenticate user
    await requireAuth(req, res, () => {
      // Check if user is admin
      if (req.user.role !== 'admin') {
        return res.status(403).json({
          success: false,
          error: 'Access denied, admin privileges required'
        });
      }
      
      next();
    });
  } catch (error) {
    logger.error('Admin authentication error:', error);
    res.status(500).json({
      success: false,
      error: 'Server error'
    });
  }
};

module.exports = {
  requireAuth,
  requireAdmin
};
