// PayPal Backend API for SupplyChain Pulse Subscription System
const express = require('express');
const router = express.Router();
const paypal = require('@paypal/checkout-server-sdk');
const db = require('../database/db');
const auth = require('../middleware/auth');
const logger = require('../utils/logger');

// PayPal client configuration
function getPayPalClient() {
  const clientId = process.env.PAYPAL_CLIENT_ID;
  const clientSecret = process.env.PAYPAL_CLIENT_SECRET;
  
  // Use sandbox environment for development, production for live
  const environment = process.env.NODE_ENV === 'production'
    ? new paypal.core.LiveEnvironment(clientId, clientSecret)
    : new paypal.core.SandboxEnvironment(clientId, clientSecret);
  
  return new paypal.core.PayPalHttpClient(environment);
}

// Process new subscription via PayPal
router.post('/process-subscription', async (req, res) => {
  try {
    const {
      orderID,
      payerID,
      payerEmail,
      payerName,
      payerCompany,
      planName,
      planPrice
    } = req.body;

    // Validate inputs
    if (!orderID || !payerID || !payerEmail || !planName) {
      return res.status(400).json({
        success: false,
        error: 'Missing required fields'
      });
    }

    // Get plan from database
    const plan = await db.plans.findOne({ name: planName.toLowerCase() });
    if (!plan) {
      return res.status(400).json({
        success: false,
        error: 'Invalid plan selected'
      });
    }

    // Verify the payment with PayPal
    const client = getPayPalClient();
    const request = new paypal.orders.OrdersGetRequest(orderID);
    const order = await client.execute(request);

    // Verify payment amount matches plan price
    const capturedAmount = parseFloat(order.result.purchase_units[0].amount.value);
    if (capturedAmount !== parseFloat(planPrice)) {
      return res.status(400).json({
        success: false,
        error: 'Payment amount does not match plan price'
      });
    }

    // Check if user already exists
    let userId;
    const existingUser = await db.users.findOne({ email: payerEmail });
    
    if (existingUser) {
      // Update existing user
      await db.users.updateOne(
        { email: payerEmail },
        { 
          $set: {
            name: payerName,
            company: payerCompany,
            paypalPayerId: payerID,
            updatedAt: new Date()
          }
        }
      );
      userId = existingUser._id;
    } else {
      // Create new user
      const result = await db.users.insertOne({
        email: payerEmail,
        name: payerName,
        company: payerCompany,
        paypalPayerId: payerID,
        role: 'customer',
        createdAt: new Date(),
        updatedAt: new Date()
      });
      userId = result.insertedId;
    }

    // Create subscription in PayPal
    // Note: For actual recurring billing with PayPal, you would use their Subscriptions API
    // This example uses Orders API for simplicity, but in production you would implement
    // the full subscription flow with webhooks
    
    // For now, we'll create a subscription record in our database
    const subscriptionId = `pp_${Date.now()}_${Math.floor(Math.random() * 1000)}`;
    const currentDate = new Date();
    const nextBillingDate = new Date();
    nextBillingDate.setMonth(nextBillingDate.getMonth() + 1); // Bill monthly
    
    // Save subscription to database
    await db.subscriptions.insertOne({
      userId: userId,
      paypalOrderId: orderID,
      paypalSubscriptionId: subscriptionId, // In production, this would be a real PayPal subscription ID
      planName: planName.toLowerCase(),
      planAmount: parseFloat(planPrice),
      status: 'active',
      paymentProcessor: 'paypal',
      currentPeriodStart: currentDate,
      currentPeriodEnd: nextBillingDate,
      createdAt: currentDate,
      updatedAt: currentDate
    });

    // Record the payment
    await db.payments.insertOne({
      userId: userId,
      subscriptionId: subscriptionId,
      paypalOrderId: orderID,
      amount: parseFloat(planPrice),
      status: 'succeeded',
      paymentProcessor: 'paypal',
      paymentDate: currentDate,
      createdAt: currentDate
    });

    // Return success response
    res.json({
      success: true,
      subscriptionId: subscriptionId,
      message: 'Subscription created successfully'
    });
  } catch (error) {
    logger.error('Error processing PayPal subscription:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to process subscription'
    });
  }
});

// Handle PayPal IPN (Instant Payment Notification)
router.post('/ipn', express.raw({ type: 'application/x-www-form-urlencoded' }), async (req, res) => {
  try {
    // Verify the IPN message with PayPal
    const body = req.body.toString();
    const verifyRequest = `cmd=_notify-validate&${body}`;
    
    // Make HTTP request to PayPal to validate
    const verifyResponse = await fetch('https://ipnpb.paypal.com/cgi-bin/webscr', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Content-Length': verifyRequest.length
      },
      body: verifyRequest
    });
    
    const verificationResult = await verifyResponse.text();
    
    if (verificationResult !== 'VERIFIED') {
      logger.error('PayPal IPN verification failed');
      return res.status(400).send('IPN Verification Failed');
    }
    
    // Parse the original IPN message
    const ipnData = new URLSearchParams(body);
    const txnType = ipnData.get('txn_type');
    
    // Handle different transaction types
    switch (txnType) {
      case 'subscr_signup':
        await handleSubscriptionSignup(ipnData);
        break;
      case 'subscr_payment':
        await handleSubscriptionPayment(ipnData);
        break;
      case 'subscr_cancel':
        await handleSubscriptionCancel(ipnData);
        break;
      case 'subscr_failed':
        await handleSubscriptionFailed(ipnData);
        break;
      default:
        logger.info(`Unhandled PayPal IPN transaction type: ${txnType}`);
    }
    
    // Return a 200 response to acknowledge receipt of the IPN
    res.status(200).send('OK');
  } catch (error) {
    logger.error('Error handling PayPal IPN:', error);
    res.status(500).send('IPN Handler Error');
  }
});

// Get subscription details
router.get('/subscription/:id', auth.requireAuth, async (req, res) => {
  try {
    const subscription = await db.subscriptions.findOne({
      paypalSubscriptionId: req.params.id,
      userId: req.user._id
    });

    if (!subscription) {
      return res.status(404).json({
        success: false,
        error: 'Subscription not found'
      });
    }

    // Get plan details
    const plan = await db.plans.findOne({ name: subscription.planName });

    res.json({
      success: true,
      subscription: {
        id: subscription.paypalSubscriptionId,
        planName: subscription.planName,
        planDisplayName: plan.displayName,
        amount: subscription.planAmount,
        status: subscription.status,
        currentPeriodStart: subscription.currentPeriodStart,
        currentPeriodEnd: subscription.currentPeriodEnd,
        features: plan.features,
        paymentProcessor: 'paypal'
      }
    });
  } catch (error) {
    logger.error('Error retrieving PayPal subscription:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to retrieve subscription'
    });
  }
});

// Cancel subscription
router.post('/subscription/:id/cancel', auth.requireAuth, async (req, res) => {
  try {
    // Verify subscription belongs to user
    const subscription = await db.subscriptions.findOne({
      paypalSubscriptionId: req.params.id,
      userId: req.user._id
    });

    if (!subscription) {
      return res.status(404).json({
        success: false,
        error: 'Subscription not found'
      });
    }

    // In production, you would call PayPal's API to cancel the subscription
    // For this example, we'll just update our database
    
    // Update subscription in database
    await db.subscriptions.updateOne(
      { paypalSubscriptionId: req.params.id },
      { 
        $set: {
          status: 'canceling',
          updatedAt: new Date()
        }
      }
    );

    res.json({
      success: true,
      message: 'Subscription will be canceled at the end of the billing period'
    });
  } catch (error) {
    logger.error('Error canceling PayPal subscription:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to cancel subscription'
    });
  }
});

// Helper functions for IPN handlers
async function handleSubscriptionSignup(ipnData) {
  try {
    const subscriptionId = ipnData.get('subscr_id');
    const payerEmail = ipnData.get('payer_email');
    const payerID = ipnData.get('payer_id');
    const firstName = ipnData.get('first_name');
    const lastName = ipnData.get('last_name');
    const amount = ipnData.get('amount3'); // Recurring amount
    
    logger.info(`PayPal subscription signup: ${subscriptionId} for ${payerEmail}`);
    
    // Find user by email
    const user = await db.users.findOne({ email: payerEmail });
    
    if (user) {
      // Check if subscription already exists
      const existingSubscription = await db.subscriptions.findOne({
        paypalSubscriptionId: subscriptionId
      });
      
      if (!existingSubscription) {
        // Determine plan based on amount
        let planName;
        if (parseFloat(amount) === 149) {
          planName = 'basic';
        } else if (parseFloat(amount) === 249) {
          planName = 'professional';
        } else if (parseFloat(amount) === 499) {
          planName = 'enterprise';
        } else {
          planName = 'custom';
        }
        
        // Create new subscription record
        const currentDate = new Date();
        const nextBillingDate = new Date();
        nextBillingDate.setMonth(nextBillingDate.getMonth() + 1); // Bill monthly
        
        await db.subscriptions.insertOne({
          userId: user._id,
          paypalSubscriptionId: subscriptionId,
          planName: planName,
          planAmount: parseFloat(amount),
          status: 'active',
          paymentProcessor: 'paypal',
          currentPeriodStart: currentDate,
          currentPeriodEnd: nextBillingDate,
          createdAt: currentDate,
          updatedAt: currentDate
        });
      }
    } else {
      logger.error(`User not found for PayPal subscription: ${payerEmail}`);
    }
  } catch (error) {
    logger.error('Error handling subscription signup:', error);
  }
}

async function handleSubscriptionPayment(ipnData) {
  try {
    const subscriptionId = ipnData.get('subscr_id');
    const payerEmail = ipnData.get('payer_email');
    const transactionId = ipnData.get('txn_id');
    const paymentStatus = ipnData.get('payment_status');
    const amount = ipnData.get('mc_gross');
    
    logger.info(`PayPal subscription payment: ${subscriptionId}, status: ${paymentStatus}`);
    
    if (paymentStatus === 'Completed') {
      // Find subscription
      const subscription = await db.subscriptions.findOne({
        paypalSubscriptionId: subscriptionId
      });
      
      if (subscription) {
        // Update subscription dates
        const currentDate = new Date();
        const nextBillingDate = new Date();
        nextBillingDate.setMonth(nextBillingDate.getMonth() + 1); // Bill monthly
        
        await db.subscriptions.updateOne(
          { paypalSubscriptionId: subscriptionId },
          { 
            $set: {
              status: 'active',
              currentPeriodStart: currentDate,
              currentPeriodEnd: nextBillingDate,
              updatedAt: currentDate
            }
          }
        );
        
        // Record payment
        await db.payments.insertOne({
          userId: subscription.userId,
          subscriptionId: subscriptionId,
          paypalTransactionId: transactionId,
          amount: parseFloat(amount),
          status: 'succeeded',
          paymentProcessor: 'paypal',
          paymentDate: currentDate,
          createdAt: currentDate
        });
      } else {
        logger.error(`Subscription not found for payment: ${subscriptionId}`);
      }
    }
  } catch (error) {
    logger.error('Error handling subscription payment:', error);
  }
}

async function handleSubscriptionCancel(ipnData) {
  try {
    const subscriptionId = ipnData.get('subscr_id');
    
    logger.info(`PayPal subscription canceled: ${subscriptionId}`);
    
    // Update subscription in database
    await db.subscriptions.updateOne(
      { paypalSubscriptionId: subscriptionId },
      { 
        $set: {
          status: 'canceled',
          canceledAt: new Date(),
          updatedAt: new Date()
        }
      }
    );
  } catch (error) {
    logger.error('Error handling subscription cancel:', error);
  }
}

async function handleSubscriptionFailed(ipnData) {
  try {
    const subscriptionId = ipnData.get('subscr_id');
    const payerEmail = ipnData.get('payer_email');
    
    logger.info(`PayPal subscription payment failed: ${subscriptionId}`);
    
    // Update subscription in database
    await db.subscriptions.updateOne(
      { paypalSubscriptionId: subscriptionId },
      { 
        $set: {
          status: 'past_due',
          updatedAt: new Date()
        }
      }
    );
    
    // Record failed payment
    const subscription = await db.subscriptions.findOne({
      paypalSubscriptionId: subscriptionId
    });
    
    if (subscription) {
      await db.payments.insertOne({
        userId: subscription.userId,
        subscriptionId: subscriptionId,
        amount: subscription.planAmount,
        status: 'failed',
        paymentProcessor: 'paypal',
        paymentDate: new Date(),
        createdAt: new Date()
      });
      
      // Send payment failure notification email
      // This would integrate with your email service
      // sendPaymentFailureEmail(payerEmail, subscriptionId);
    }
  } catch (error) {
    logger.error('Error handling subscription payment failure:', error);
  }
}

module.exports = router;
