// Main Server Application for SupplyChain Pulse Subscription System
const express = require('express');
const cors = require('cors');
const helmet = require('helmet');
const morgan = require('morgan');
const path = require('path');
const db = require('./database/db');
const logger = require('./utils/logger');

// Load environment variables
require('dotenv').config();

// Initialize Express app
const app = express();
const PORT = process.env.PORT || 3000;

// Middleware
app.use(helmet()); // Security headers
app.use(cors()); // Enable CORS
app.use(express.json()); // Parse JSON bodies
app.use(express.urlencoded({ extended: true })); // Parse URL-encoded bodies
app.use(morgan('combined')); // HTTP request logging

// Serve static files
app.use(express.static(path.join(__dirname, 'public')));

// API routes
app.use('/api/users', require('./user_management/user-api'));
app.use('/api/stripe', require('./stripe/stripe-api'));
app.use('/api/paypal', require('./paypal/paypal-api'));

// Admin routes
app.use('/api/admin', require('./admin/admin-api'));

// Serve subscription page
app.get('/subscription', (req, res) => {
  res.sendFile(path.join(__dirname, 'public', 'subscription.html'));
});

// Serve subscription success page
app.get('/subscription/success', (req, res) => {
  res.sendFile(path.join(__dirname, 'public', 'subscription-success.html'));
});

// Serve subscription cancel page
app.get('/subscription/cancel', (req, res) => {
  res.sendFile(path.join(__dirname, 'public', 'subscription-cancel.html'));
});

// Error handling middleware
app.use((err, req, res, next) => {
  logger.error('Server error:', err);
  res.status(500).json({
    success: false,
    error: 'Server error'
  });
});

// Start server
async function startServer() {
  try {
    // Connect to database
    await db.connect();
    
    // Start listening
    app.listen(PORT, () => {
      logger.info(`Server running on port ${PORT}`);
    });
  } catch (error) {
    logger.error('Failed to start server:', error);
    process.exit(1);
  }
}

// Start the server
startServer();
