// Stripe Backend API for SupplyChain Pulse Subscription System
const express = require('express');
const router = express.Router();
const stripe = require('stripe')(process.env.STRIPE_SECRET_KEY);
const db = require('../database/db');
const auth = require('../middleware/auth');
const logger = require('../utils/logger');

// Create subscription plans in Stripe (run once during setup)
router.post('/create-plans', auth.requireAdmin, async (req, res) => {
  try {
    // Create Basic plan
    const basicPlan = await stripe.products.create({
      name: 'Basic Plan',
      description: 'SupplyChain Pulse Basic Subscription'
    });
    
    const basicPrice = await stripe.prices.create({
      product: basicPlan.id,
      unit_amount: 14900, // $149.00
      currency: 'usd',
      recurring: {
        interval: 'month'
      },
      metadata: {
        plan_name: 'basic'
      }
    });

    // Create Professional plan
    const professionalPlan = await stripe.products.create({
      name: 'Professional Plan',
      description: 'SupplyChain Pulse Professional Subscription'
    });
    
    const professionalPrice = await stripe.prices.create({
      product: professionalPlan.id,
      unit_amount: 24900, // $249.00
      currency: 'usd',
      recurring: {
        interval: 'month'
      },
      metadata: {
        plan_name: 'professional'
      }
    });

    // Create Enterprise plan
    const enterprisePlan = await stripe.products.create({
      name: 'Enterprise Plan',
      description: 'SupplyChain Pulse Enterprise Subscription'
    });
    
    const enterprisePrice = await stripe.prices.create({
      product: enterprisePlan.id,
      unit_amount: 49900, // $499.00
      currency: 'usd',
      recurring: {
        interval: 'month'
      },
      metadata: {
        plan_name: 'enterprise'
      }
    });

    // Save price IDs to database for future reference
    await db.plans.insertMany([
      {
        name: 'basic',
        displayName: 'Basic',
        priceId: basicPrice.id,
        amount: 149,
        features: [
          'Real-time disruption monitoring',
          'Global risk assessment',
          'Daily email alerts',
          'Basic reporting tools',
          '30-day data history'
        ]
      },
      {
        name: 'professional',
        displayName: 'Professional',
        priceId: professionalPrice.id,
        amount: 249,
        features: [
          'All Basic features',
          'Industry-specific alerts',
          'Advanced analytics dashboard',
          'Custom risk assessment reports',
          '90-day data history',
          'API access'
        ]
      },
      {
        name: 'enterprise',
        displayName: 'Enterprise',
        priceId: enterprisePrice.id,
        amount: 499,
        features: [
          'All Professional features',
          'Dedicated account manager',
          'Custom integration with ERP systems',
          'Unlimited data history',
          'Priority support',
          'Quarterly strategy consultations'
        ]
      }
    ]);

    res.json({
      success: true,
      plans: {
        basic: basicPrice.id,
        professional: professionalPrice.id,
        enterprise: enterprisePrice.id
      }
    });
  } catch (error) {
    logger.error('Error creating subscription plans:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to create subscription plans'
    });
  }
});

// Process new subscription
router.post('/process-subscription', async (req, res) => {
  try {
    const {
      paymentMethodId,
      customerName,
      customerEmail,
      customerCompany,
      planName,
      planPrice
    } = req.body;

    // Validate inputs
    if (!paymentMethodId || !customerEmail || !planName) {
      return res.status(400).json({
        success: false,
        error: 'Missing required fields'
      });
    }

    // Get plan price ID from database
    const plan = await db.plans.findOne({ name: planName.toLowerCase() });
    if (!plan) {
      return res.status(400).json({
        success: false,
        error: 'Invalid plan selected'
      });
    }

    // Check if customer already exists
    let customer;
    const existingCustomer = await db.users.findOne({ email: customerEmail });
    
    if (existingCustomer && existingCustomer.stripeCustomerId) {
      // Use existing customer
      customer = await stripe.customers.retrieve(existingCustomer.stripeCustomerId);
      
      // Update payment method
      await stripe.paymentMethods.attach(paymentMethodId, {
        customer: customer.id
      });
      
      // Set as default payment method
      await stripe.customers.update(customer.id, {
        invoice_settings: {
          default_payment_method: paymentMethodId
        }
      });
    } else {
      // Create new customer
      customer = await stripe.customers.create({
        email: customerEmail,
        name: customerName,
        payment_method: paymentMethodId,
        invoice_settings: {
          default_payment_method: paymentMethodId
        },
        metadata: {
          company: customerCompany
        }
      });
      
      // Create or update user in database
      if (existingCustomer) {
        await db.users.updateOne(
          { email: customerEmail },
          { 
            $set: {
              stripeCustomerId: customer.id,
              name: customerName,
              company: customerCompany,
              updatedAt: new Date()
            }
          }
        );
      } else {
        // Create new user
        await db.users.insertOne({
          email: customerEmail,
          name: customerName,
          company: customerCompany,
          stripeCustomerId: customer.id,
          role: 'customer',
          createdAt: new Date(),
          updatedAt: new Date()
        });
      }
    }

    // Create the subscription
    const subscription = await stripe.subscriptions.create({
      customer: customer.id,
      items: [
        { price: plan.priceId }
      ],
      payment_behavior: 'default_incomplete',
      expand: ['latest_invoice.payment_intent'],
      metadata: {
        planName: planName.toLowerCase()
      }
    });

    // Save subscription to database
    await db.subscriptions.insertOne({
      userId: existingCustomer ? existingCustomer._id : (await db.users.findOne({ email: customerEmail }))._id,
      stripeSubscriptionId: subscription.id,
      planName: planName.toLowerCase(),
      planAmount: planPrice,
      status: subscription.status,
      currentPeriodStart: new Date(subscription.current_period_start * 1000),
      currentPeriodEnd: new Date(subscription.current_period_end * 1000),
      createdAt: new Date(),
      updatedAt: new Date()
    });

    // Return the client secret for the payment intent
    res.json({
      success: true,
      subscriptionId: subscription.id,
      clientSecret: subscription.latest_invoice.payment_intent.client_secret
    });
  } catch (error) {
    logger.error('Error processing subscription:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to process subscription'
    });
  }
});

// Handle subscription webhook events from Stripe
router.post('/webhook', express.raw({ type: 'application/json' }), async (req, res) => {
  const sig = req.headers['stripe-signature'];
  let event;

  try {
    event = stripe.webhooks.constructEvent(
      req.body,
      sig,
      process.env.STRIPE_WEBHOOK_SECRET
    );
  } catch (err) {
    logger.error('Webhook signature verification failed:', err);
    return res.status(400).send(`Webhook Error: ${err.message}`);
  }

  // Handle the event
  switch (event.type) {
    case 'customer.subscription.created':
      await handleSubscriptionCreated(event.data.object);
      break;
    case 'customer.subscription.updated':
      await handleSubscriptionUpdated(event.data.object);
      break;
    case 'customer.subscription.deleted':
      await handleSubscriptionCanceled(event.data.object);
      break;
    case 'invoice.payment_succeeded':
      await handleInvoicePaymentSucceeded(event.data.object);
      break;
    case 'invoice.payment_failed':
      await handleInvoicePaymentFailed(event.data.object);
      break;
    default:
      // Unexpected event type
      logger.info(`Unhandled event type: ${event.type}`);
  }

  // Return a 200 response to acknowledge receipt of the event
  res.send();
});

// Get subscription details
router.get('/subscription/:id', auth.requireAuth, async (req, res) => {
  try {
    const subscription = await db.subscriptions.findOne({
      stripeSubscriptionId: req.params.id,
      userId: req.user._id
    });

    if (!subscription) {
      return res.status(404).json({
        success: false,
        error: 'Subscription not found'
      });
    }

    // Get plan details
    const plan = await db.plans.findOne({ name: subscription.planName });

    res.json({
      success: true,
      subscription: {
        id: subscription.stripeSubscriptionId,
        planName: subscription.planName,
        planDisplayName: plan.displayName,
        amount: subscription.planAmount,
        status: subscription.status,
        currentPeriodStart: subscription.currentPeriodStart,
        currentPeriodEnd: subscription.currentPeriodEnd,
        features: plan.features
      }
    });
  } catch (error) {
    logger.error('Error retrieving subscription:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to retrieve subscription'
    });
  }
});

// Cancel subscription
router.post('/subscription/:id/cancel', auth.requireAuth, async (req, res) => {
  try {
    // Verify subscription belongs to user
    const subscription = await db.subscriptions.findOne({
      stripeSubscriptionId: req.params.id,
      userId: req.user._id
    });

    if (!subscription) {
      return res.status(404).json({
        success: false,
        error: 'Subscription not found'
      });
    }

    // Cancel at period end
    await stripe.subscriptions.update(req.params.id, {
      cancel_at_period_end: true
    });

    // Update subscription in database
    await db.subscriptions.updateOne(
      { stripeSubscriptionId: req.params.id },
      { 
        $set: {
          status: 'canceling',
          updatedAt: new Date()
        }
      }
    );

    res.json({
      success: true,
      message: 'Subscription will be canceled at the end of the billing period'
    });
  } catch (error) {
    logger.error('Error canceling subscription:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to cancel subscription'
    });
  }
});

// Change subscription plan
router.post('/subscription/:id/change-plan', auth.requireAuth, async (req, res) => {
  try {
    const { newPlanName } = req.body;

    // Validate inputs
    if (!newPlanName) {
      return res.status(400).json({
        success: false,
        error: 'Missing required fields'
      });
    }

    // Verify subscription belongs to user
    const subscription = await db.subscriptions.findOne({
      stripeSubscriptionId: req.params.id,
      userId: req.user._id
    });

    if (!subscription) {
      return res.status(404).json({
        success: false,
        error: 'Subscription not found'
      });
    }

    // Get new plan price ID
    const newPlan = await db.plans.findOne({ name: newPlanName.toLowerCase() });
    if (!newPlan) {
      return res.status(400).json({
        success: false,
        error: 'Invalid plan selected'
      });
    }

    // Get subscription from Stripe
    const stripeSubscription = await stripe.subscriptions.retrieve(req.params.id);

    // Update subscription with new price
    await stripe.subscriptions.update(req.params.id, {
      items: [
        {
          id: stripeSubscription.items.data[0].id,
          price: newPlan.priceId
        }
      ],
      proration_behavior: 'create_prorations'
    });

    // Update subscription in database
    await db.subscriptions.updateOne(
      { stripeSubscriptionId: req.params.id },
      { 
        $set: {
          planName: newPlanName.toLowerCase(),
          planAmount: newPlan.amount,
          updatedAt: new Date()
        }
      }
    );

    res.json({
      success: true,
      message: 'Subscription plan updated successfully'
    });
  } catch (error) {
    logger.error('Error changing subscription plan:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to change subscription plan'
    });
  }
});

// Update payment method
router.post('/update-payment-method', auth.requireAuth, async (req, res) => {
  try {
    const { paymentMethodId } = req.body;

    // Validate inputs
    if (!paymentMethodId) {
      return res.status(400).json({
        success: false,
        error: 'Missing required fields'
      });
    }

    // Get customer ID
    const user = await db.users.findOne({ _id: req.user._id });
    if (!user || !user.stripeCustomerId) {
      return res.status(404).json({
        success: false,
        error: 'Customer not found'
      });
    }

    // Attach payment method to customer
    await stripe.paymentMethods.attach(paymentMethodId, {
      customer: user.stripeCustomerId
    });

    // Set as default payment method
    await stripe.customers.update(user.stripeCustomerId, {
      invoice_settings: {
        default_payment_method: paymentMethodId
      }
    });

    res.json({
      success: true,
      message: 'Payment method updated successfully'
    });
  } catch (error) {
    logger.error('Error updating payment method:', error);
    res.status(500).json({
      success: false,
      error: 'Failed to update payment method'
    });
  }
});

// Helper functions for webhook handlers
async function handleSubscriptionCreated(subscription) {
  try {
    logger.info(`Subscription created: ${subscription.id}`);
    // Additional logic if needed
  } catch (error) {
    logger.error('Error handling subscription created:', error);
  }
}

async function handleSubscriptionUpdated(subscription) {
  try {
    logger.info(`Subscription updated: ${subscription.id}`);
    
    // Update subscription in database
    await db.subscriptions.updateOne(
      { stripeSubscriptionId: subscription.id },
      { 
        $set: {
          status: subscription.status,
          currentPeriodStart: new Date(subscription.current_period_start * 1000),
          currentPeriodEnd: new Date(subscription.current_period_end * 1000),
          updatedAt: new Date()
        }
      }
    );
  } catch (error) {
    logger.error('Error handling subscription updated:', error);
  }
}

async function handleSubscriptionCanceled(subscription) {
  try {
    logger.info(`Subscription canceled: ${subscription.id}`);
    
    // Update subscription in database
    await db.subscriptions.updateOne(
      { stripeSubscriptionId: subscription.id },
      { 
        $set: {
          status: 'canceled',
          canceledAt: new Date(),
          updatedAt: new Date()
        }
      }
    );
  } catch (error) {
    logger.error('Error handling subscription canceled:', error);
  }
}

async function handleInvoicePaymentSucceeded(invoice) {
  try {
    logger.info(`Invoice payment succeeded: ${invoice.id}`);
    
    if (invoice.subscription) {
      // Update subscription in database
      await db.subscriptions.updateOne(
        { stripeSubscriptionId: invoice.subscription },
        { 
          $set: {
            status: 'active',
            updatedAt: new Date()
          }
        }
      );
      
      // Record payment
      await db.payments.insertOne({
        userId: (await db.subscriptions.findOne({ stripeSubscriptionId: invoice.subscription })).userId,
        subscriptionId: invoice.subscription,
        invoiceId: invoice.id,
        amount: invoice.amount_paid / 100, // Convert from cents
        status: 'succeeded',
        paymentDate: new Date(invoice.created * 1000),
        createdAt: new Date()
      });
    }
  } catch (error) {
    logger.error('Error handling invoice payment succeeded:', error);
  }
}

async function handleInvoicePaymentFailed(invoice) {
  try {
    logger.info(`Invoice payment failed: ${invoice.id}`);
    
    if (invoice.subscription) {
      // Update subscription in database
      await db.subscriptions.updateOne(
        { stripeSubscriptionId: invoice.subscription },
        { 
          $set: {
            status: 'past_due',
            updatedAt: new Date()
          }
        }
      );
      
      // Record failed payment
      await db.payments.insertOne({
        userId: (await db.subscriptions.findOne({ stripeSubscriptionId: invoice.subscription })).userId,
        subscriptionId: invoice.subscription,
        invoiceId: invoice.id,
        amount: invoice.amount_due / 100, // Convert from cents
        status: 'failed',
        paymentDate: new Date(invoice.created * 1000),
        createdAt: new Date()
      });
      
      // Get user email for notification
      const subscription = await db.subscriptions.findOne({ stripeSubscriptionId: invoice.subscription });
      const user = await db.users.findOne({ _id: subscription.userId });
      
      // Send payment failure notification email
      // This would integrate with your email service
      // sendPaymentFailureEmail(user.email, invoice.id);
    }
  } catch (error) {
    logger.error('Error handling invoice payment failed:', error);
  }
}

module.exports = router;
